

SSH_MAPPER_DICT = {
    "aruba_os": {
        "cmd": "show version",
        "search_patterns": [r"ArubaOS"]
    },
    "alcatel_aos": {
        "cmd": "show system",
        "search_patterns": [r"Alcatel-Lucent"]
    },
    "alcatel_sros": {
        "cmd": "show version",
        "search_patterns": ["Nokia", "Alcatel"]
    },
    "cisco_ios": {
        "cmd": "show version",
        "search_patterns": [
            "Cisco IOS Software",
            "Cisco Internetwork Operating System Software",
        ]
    },
    "cisco_nxos": {
        "cmd": "show version",
        "search_patterns": [r"Cisco Nexus Operating System", r"NX-OS"],
    },
    "cisco_xr": {
        "cmd": "show version",
        "search_patterns": [r"Cisco IOS XR"],
    },
    "hp_comware": {
        "cmd": "display version",
        "search_patterns": ["HPE Comware", "HP Comware"],
    },
    "juniper_junos": {
        "cmd": "show version",
        "search_patterns": [
            r"JUNOS Software Release",
            r"JUNOS .+ Software",
            r"JUNOS OS Kernel",
            r"JUNOS Base Version",
        ],
    },
}


#!/usr/bin/env python

import paramiko
import time


"""
A generic and agnostic module to establish SSH connection to any devices that supports SSH.
"""


conf_terminal_setup = f"configure terminal"

def ssh_connector(hostname, username, password, key=False, timeout=10, port=22):
    """ Connect to remote device and return a channel to use for sending cmds.
        return the returned value is the channel object that will be used to send command to remote device
    """
    ssh = paramiko.SSHClient()
    try:
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        ssh.connect(hostname=hostname, port=port, username=username,
                    password=password, look_for_keys=key, timeout=timeout)
        print("Connected to {0}\n".format(hostname))
    except:
        print("Could not connect to {0}".format(hostname))
        ssh.close()
        return None
    else:
        channel = ssh.invoke_shell()
        return channel


def send_cmd(cmd, channel, incoming_sleep_time=2, out_going_sleep_time=2):
    """Send a cmd in 'global configuration mode' via the channel if channel object is not None
        return: the return value is the result or the executed cmd
    """

    if not cmd:
        print(f"Not command to send\n")
        return None
    if not channel:
        print(f"No channel available\n")
        return None

    banner = channel.recv(99999).decode("utf-8")
    time.sleep(incoming_sleep_time)
    print(f"{banner}\n\n")

    channel.send(cmd + "\n")
    time.sleep(out_going_sleep_time)

    output = channel.recv(99999).decode("utf-8")
    time.sleep(incoming_sleep_time)
    return output


def send_config_cmd(cmd, channel, incoming_sleep_time=2, out_going_sleep_time=2):
    """Send a cmd in 'configuration mode' via the channel if channel object is not None
        return: the return value is the result or the executed cmd
    """

    if not cmd:
        print(f"Not command to send\n")
        return None
    if not channel:
        print(f"No channel available\n")
        return None

    banner = channel.recv(99999).decode("utf-8")
    time.sleep(incoming_sleep_time)
    print(f"{banner}\n\n")

    # Set the config terminal
    channel.send(conf_terminal_setup + "\n")
    time.sleep(out_going_sleep_time)
    tmp_output = channel.recv(99999).decode("utf-8")
    time.sleep(incoming_sleep_time)
    print(tmp_output)

    channel.send(cmd + "\n")
    time.sleep(out_going_sleep_time)

    output = channel.recv(99999).decode("utf-8")
    time.sleep(incoming_sleep_time)
    return output


def send_cmds(list_of_commands, channel, incoming_sleep_time=2, out_going_sleep_time=2):
    """Send multiple cmds in 'global configuration mode' via the channel if channel object is not None
        return: the return value is the result or the executed cmds
    """
    list_of_commands = list(list_of_commands)

    if not list_of_commands:
        print(f"Not commands to send\n")
        return None
    if not channel:
        print(f"No channel available\n")
        return None

    banner = channel.recv(99999).decode("utf-8")
    time.sleep(incoming_sleep_time)
    print(f"{banner}\n\n")

    all_outputs = ""
    for cmd in list_of_commands:
        channel.send(cmd + "\n")
        time.sleep(out_going_sleep_time)

        output = channel.recv(99999).decode("utf-8")
        time.sleep(incoming_sleep_time)
        all_outputs += f"{output}\n"

    #channel.close()
    return all_outputs


def send_config_cmds(list_of_commands, channel, incoming_sleep_time=2, out_going_sleep_time=2):
    """Send multiple cmds in 'configuration mode' via the channel if channel object is not None
        return: the return value is the result or the executed cmds
    """
    list_of_commands = list(list_of_commands)

    if not list_of_commands:
        print(f"Not commands to send\n")
        return None
    if not channel:
        print(f"No channel available\n")
        return None

    banner = channel.recv(99999).decode("utf-8")
    time.sleep(incoming_sleep_time)
    print(f"{banner}\n\n")

    # Set the config terminal
    channel.send(conf_terminal_setup + "\n")
    time.sleep(out_going_sleep_time)
    tmp_output = channel.recv(99999).decode("utf-8")
    time.sleep(incoming_sleep_time)
    print(tmp_output)

    all_outputs = ""
    for cmd in list_of_commands:
        channel.send(cmd + "\n")
        time.sleep(out_going_sleep_time)

        output = channel.recv(99999).decode("utf-8")
        time.sleep(incoming_sleep_time)
        all_outputs += f"{output}\n"

    #channel.close()
    return all_outputs


# This function is not yet tested, I am working on it.

# def send_commands_return_dictionary(list_of_commands, list_of_hosts, username,
#                                     password, incoming_sleep_time=1, out_going_sleep_time=1):
#     """connect to each device and execute all commands per device and record the results per device in a dict"""
#     output_dict = {}
#     if not list_of_commands:
#         print(f"Not commands to send\n")
#         return None
#     if not list_of_hosts:
#         print(f"Not hosts in the list of hosts\n")
#         return None
#
#     for host in list_of_hosts:
#         new_channel = ssh_connector(hostname=host, username=username, password=password)
#
#         all_outputs = ""
#         for cmd in list_of_commands:
#             new_channel.send(cmd + "\n")
#             time.sleep(out_going_sleep_time)
#
#             output = new_channel.recv(99999).decode("utf-8")
#             time.sleep(incoming_sleep_time)
#             all_outputs += f"{output}\n"
#
#         output_dict[host] = all_outputs
#         all_outputs = ""
#         new_channel.close()
#
#     return output_dict
























import os
from setuptools import setup

def read(fname):
    with open("README.md") as f:
        return f.read()

setup(
    name = "sshFRIEND",
    version = "0.0.9",
    author = "David Johnnes",
    author_email = "david.johnnes@gmail.com",
    description = ("A generic and platform agnostic SSH module to access and send commands to remote devices that support OpenSSH"),
    license = "BSD",
    keywords = "ssh access, ssh remote command execution",
    url = "",
    packages=['sshFRIEND'],
    long_description=read('README.md'),
    classifiers=[
        "Topic :: Utilities",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: BSD License",
    ],
)